/**CHeaderFile*****************************************************************

  FileName    [bmcInt.h]

  PackageName [bmc]

  Synopsis    [The internal header file for the <tt>bmc</tt> package]

  Description []

  SeeAlso     []

  Author      [Roberto Cavada]

  Copyright   [
  This file is part of the ``bmc'' package of NuSMV version 2. 
  Copyright (C) 2000-2001 by ITC-irst and University of Trento. 

  NuSMV version 2 is free software; you can redistribute it and/or 
  modify it under the terms of the GNU Lesser General Public 
  License as published by the Free Software Foundation; either 
  version 2 of the License, or (at your option) any later version.

  NuSMV version 2 is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public 
  License along with this library; if not, write to the Free Software 
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.

  For more information of NuSMV see <http://nusmv.irst.itc.it>
  or email to <nusmv-users@irst.itc.it>.
  Please report bugs to <nusmv-users@irst.itc.it>.

  To contact the NuSMV development board, email to <nusmv@irst.itc.it>. ]

******************************************************************************/

#ifndef _BMC_INT_H
#define _BMC_INT_H

#include <time.h>
#include <limits.h>
#include "util.h" 
#include "utils.h"
#include "ucmd.h" 
#include "node.h" 
#include "grammar.h" 
#include "set.h" 
#include "dd.h" 
#include "compile.h" 
#include "prop.h"
#include "opt.h" 
#include "sim.h"
#include "cmd.h"
#include "mc.h"
#include "parser.h"
#include "st.h" /* hash tables are used in memoizing operations */
#include "bmc.h" 

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/* SAT resolver interface: */
typedef enum sat_result_TAG
{ SAT_INTERNAL_ERROR=-1, 
  SAT_SATISFIABLE_PROBLEM, 
  SAT_UNSATISFIABLE_PROBLEM, 
  SAT_UNAVAILABLE
} sat_result;


EXTERN sat_result Bmc_CallSatoSolver ARGS((char * fname));
EXTERN sat_result Bmc_CallSimSolver  ARGS((lsList CnfClauses, lsList CnfVars, 
					   int CnfMaxVar, lsList lsResult));


/* matching function for iteration in lists of wffs */
typedef int (*BMC_PF_MATCH)(node_ptr,  int, void*);

/* answer function in case of match: */  
typedef void (*BMC_PF_MATCH_ANSWER)(node_ptr, int, void*);


/*---------------------------------------------------------------------------*/
/* Structure declarations                                                    */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

EXTERN FILE * nusmv_stderr;
EXTERN FILE * nusmv_stdout;
EXTERN cmp_struct_ptr cmps;
EXTERN node_ptr all_variables;
EXTERN node_ptr all_symbols;
EXTERN options_ptr options;
EXTERN DdManager * dd_manager;


/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

#ifndef max
#define max(_a_, _b_) ((_a_ < _b_) ? _b_ : _a_)
#endif

#ifndef min
#define min(_a_, _b_) ((_a_ < _b_) ? _a_ : _b_)
#endif

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

/* constructor and destructor of the Variables Manager: */
EXTERN void BmcVarsMgrInit ARGS((node_ptr state_vars_bexp));
EXTERN void BmcVarsMgrQuit ARGS((void));

EXTERN char * Bmc_StripPath ARGS((char * pathfname));
EXTERN void Bmc_StripPathExt ARGS((char * fpathname, char* filename));
EXTERN void BmcIntFreeListOfLists ARGS((lsList list_of_lists));
 
/* RC EXTERN node_ptr eval_bexp ARGS((node_ptr bexp)); */

EXTERN node_ptr Bmc_WffMkBinary ARGS((int type, node_ptr arg1, node_ptr arg2));
EXTERN node_ptr Bmc_WffMkUnary    ARGS((int type, node_ptr arg));
EXTERN node_ptr Bmc_WffMkConst    ARGS((int type));
EXTERN node_ptr Bmc_WffMkTruth    ARGS((void));
EXTERN node_ptr Bmc_WffMkFalsity  ARGS((void));
EXTERN node_ptr Bmc_WffMkNot      ARGS((node_ptr arg));
EXTERN node_ptr Bmc_WffMkAnd      ARGS((node_ptr arg1, node_ptr arg2));
EXTERN node_ptr Bmc_WffMkOr       ARGS((node_ptr arg1, node_ptr arg2));
EXTERN node_ptr Bmc_WffMkImplies  ARGS((node_ptr arg1, node_ptr arg2));
EXTERN node_ptr Bmc_WffMkIff      ARGS((node_ptr arg1, node_ptr arg2));
EXTERN node_ptr Bmc_WffMkNext     ARGS((node_ptr arg));
EXTERN node_ptr Bmc_WffMkXNext    ARGS((node_ptr arg, int x));
EXTERN node_ptr Bmc_WffMkOpNext   ARGS((node_ptr arg));
EXTERN node_ptr Bmc_WffMkXopNext  ARGS((node_ptr arg, int x));
EXTERN node_ptr Bmc_WffMkGlobally ARGS((node_ptr arg));
EXTERN node_ptr Bmc_WffMkEventually ARGS((node_ptr arg));
EXTERN node_ptr Bmc_WffMkUntil      ARGS((node_ptr arg1, node_ptr arg2));
EXTERN node_ptr Bmc_WffMkReleases   ARGS((node_ptr arg1, node_ptr arg2));
EXTERN node_ptr Bmc_LtlWffMkNnf     ARGS((node_ptr wff, boolean pol));
EXTERN node_ptr Bmc_LtlWffMkNnfToplevel ARGS((node_ptr wff));
EXTERN rbc_ptr Bmc_MkTableau_k_l ARGS((node_ptr wff, int k, int l));

EXTERN int  Bmc_TestTableau  ARGS((int argc, char ** argv));
EXTERN void Bmc_TestReset    ARGS((void));

/* ------------------------------------------------------------------------- */
/*                         Interface to Variables Manager:                   */
/* ------------------------------------------------------------------------- */


/* class getters and setters: */
EXTERN int            Bmc_VarsMgrGetNum ARGS((void));
EXTERN int            Bmc_VarsMgrGetMaxtime ARGS((void)); 
EXTERN RbcManager_ptr Bmc_VarsMgrGetRbcMgr  ARGS((void));


/* shift related operations: */
EXTERN rbc_ptr Bmc_CurrShiftToNext  ARGS((rbc_ptr exp));
EXTERN rbc_ptr Bmc_CurrNextShiftAtTime   ARGS((rbc_ptr exp, int time));
EXTERN rbc_ptr Bmc_CurrNextShiftAtTimes  ARGS((rbc_ptr exp, int ctime, 
					       int ntime));

EXTERN rbc_ptr Bmc_MkAndCurrNextInterval ARGS((rbc_ptr exp, int from, int to));
EXTERN rbc_ptr Bmc_MkOrCurrNextInterval  ARGS((rbc_ptr exp, int from, int to));


/* conversion operations node<->rbc: */
EXTERN rbc_ptr  Bmc_Bexp2Rbc           ARGS((node_ptr bexp));
EXTERN node_ptr Bmc_MapBexp2Rbc       ARGS((node_ptr bexp_list)); 

EXTERN rbc_ptr  Bmc_Statevar2Curvar    ARGS((node_ptr statevar));
EXTERN node_ptr Bmc_Curvar2Statevar    ARGS((rbc_ptr curvar));

EXTERN rbc_ptr  Bmc_Statevar2Nextvar   ARGS((node_ptr statevar));
EXTERN node_ptr Bmc_Nextvar2Statevar   ARGS((rbc_ptr nextvar));

EXTERN rbc_ptr  Bmc_Statevar2Timedvar  ARGS((node_ptr statevar, int time));
EXTERN node_ptr Bmc_Timedvar2Statevar  ARGS((rbc_ptr timedvar));


/* conversion operations rbc->rbc: */
EXTERN rbc_ptr  Bmc_Curvar2Nextvar    ARGS((rbc_ptr curvar));
EXTERN rbc_ptr  Bmc_Nextvar2Curvar    ARGS((rbc_ptr nextvar));

EXTERN rbc_ptr  Bmc_Curvar2Timedvar   ARGS((rbc_ptr curvar, int time));
EXTERN rbc_ptr  Bmc_Timedvar2Curvar   ARGS((rbc_ptr timedvar));

EXTERN rbc_ptr  Bmc_Nextvar2Timedvar  ARGS((rbc_ptr nextvar, int time));
EXTERN rbc_ptr  Bmc_Timedvar2Nextvar  ARGS((rbc_ptr timedvar));


/* time related operations: */
EXTERN void     Bmc_ExtendsMaxtime ARGS((int new_maxtime));
EXTERN node_ptr Bmc_GetIthStatevar ARGS((int i));
EXTERN rbc_ptr  Bmc_GetIthCurvar   ARGS((int i));
EXTERN rbc_ptr  Bmc_GetIthNextvar  ARGS((int i));
EXTERN rbc_ptr  Bmc_GetIthTimedvar ARGS((int i, int time));
EXTERN int      Bmc_Time2Firstvar  ARGS((int time));
EXTERN int      Bmc_VarIndex2Time  ARGS((int varIdx));
EXTERN int      Bmc_VarIndex2StateVarIndex ARGS((int varIdx));


/* ------------------------------------------------------------------------- */
/*  Interface to useful check functions (temporary located here in this pkg) */
/* ------------------------------------------------------------------------- */
EXTERN node_ptr Bmc_CheckFairnessListForPropositionalFormulae 
         ARGS((node_ptr wffList));

EXTERN int Bmc_IsPropositionalFormula ARGS((node_ptr wff));


EXTERN int Bmc_WffListMatchProperty ARGS(( node_ptr wffList, 
				    BMC_PF_MATCH pCheck,
				    void* pCheckOptArgument, 
				    int  iMaxMatches, 
				    unsigned int* aiMatchedIndexes, 
				    BMC_PF_MATCH_ANSWER pAnswer,
				    void* pAnswerOptArgument
				 ));



/* ------------------------------------------------------------------------- */
/*  dumping functionalities                                                  */
/* ------------------------------------------------------------------------- */
EXTERN outcome Bmc_DimacsWriteProblem ARGS((char * fname, int k, 
					    lsList CnfClauses, lsList CnfVars,
					    int CnfMaxVar));

EXTERN outcome Bmc_DavinciWriteProblem ARGS((rbc_ptr rbc, char * fname)); 


/* ------------------------------------------------------------------------- */
/*  other functionalities                                                    */
/* ------------------------------------------------------------------------- */
EXTERN void 
Bmc_PrintStats ARGS((RbcManager_ptr rbcManager, int clustSize, FILE* outFile));

EXTERN int Bmc_GetLtlWffDepth ARGS((node_ptr ltl_wff));

EXTERN void BmcShiftingHashInit ARGS((void));
EXTERN void BmcShiftingHashQuit ARGS((void));


/**AutomaticEnd***************************************************************/

#endif /* _BMC_INT_H */

























